/*
 Interface mqtt, funcionant en un ESP-03

 Iniciat Febrer del 2016 a partir del "Basic ESP8266 MQTT example"

 Ver 1.0  17-Set-2016     Primera versió
 
 

 CANAL SERIE:
 
 Rx:  (rebem ordres pel canal serie)
 :publish;topic;payload   // publica això:       client.publish(topic,payload);
 :subscribe;topic         // subscriu-te a això: client.subscribe(topic);

 Exemple:
 :subscribe;domotic/Bolet11/Temp

 
 Tx: (enviem pel canal serie)
 :status;Start				// programa iniciat
 :status;WiFi_ON			// WiFi iniciat
 :status;MQTT_ON			// Connectat al broker
 :status;MQTT_ERR			// Intent de connexió fallit
 :status;Error        		// Ordre pel canal sèrie no reconeguda
 :status;Ping60       		// Cada minut, per indicar que estic viu
 :callback;topic;payload    // Ha arribat això

 
Clock subscriu: (per comanda)
--------------
domotic/News        // Empíricamnet, es pot enviar un text de 112 caràcters com a màxim !!!!  (sembla que el topic també suma fins al màxim de 128, MQTT_MAX_PACKET_SIZE)
domotic/Bolet11/Temp

Clock publica: (automaticament)
-------------
clock/ping60				// Cada 60 segons envia un ping
clock/MoreNews              // Perque s'envii una notícia nova

*/

#include <ESP8266WiFi.h>
#include <PubSubClient.h>

// Canal sèrie
#define CR       0x0D
#define LF       0x0A

char  sROrdre[48];   // cadena que arriba pel canal serie
byte  pROrdre=0;

// Update these with values suitable for your network.
const char* ssid = "WLAN_99F1";
const char* password = "holaquetalvas";
const char* mqtt_server = "192.168.1.104";

// La meva adreça
IPAddress IP_addr(192, 168, 1, 71);
IPAddress IP_mask(255, 255, 255, 0);
IPAddress IP_gway(192, 168, 1, 1);

WiFiClient espClient;
PubSubClient client(espClient);

long ping_time=0;   // per contar temps del ping
int  ping_val=0;	// valor que s'envia amb el ping
char cadena[150];	// cadena de us general
int  error;

void setup() {
  Serial.begin(9600);
  
  Serial.println(":status;Start");
  
  setup_wifi();
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);
}

void setup_wifi() {
  delay(10);
  // We start by connecting to a WiFi network
  WiFi.config(IP_addr,IP_gway,IP_mask);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
  }

  Serial.println(":status;WiFi_ON");
}

// Aqui arriben els missatges MQTT que venen pel wifi
void callback(char* topic, byte* payload, unsigned int length) {
  payload[length]=0;
  sprintf(cadena,":callback;%s;%s",topic,payload);
  Serial.println(cadena);
}

void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    // Attempt to connect
    if (client.connect("ESP8266Client")) {
      Serial.println(":status;MQTT_ON");
    } else {
      error=client.state ();
      sprintf(cadena,":status;MQTT_ERR;%d",error);
      Serial.println(cadena);
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}


void loop() {
  char Rchar;
  char cmd[15],topic[30],payload[130];   // tenim definit MQTT_MAX_PACKET_SIZE = 128
  char *sep1,*sep2;
  byte msg_ok;

  // Gestió de la connexió al sevidor
  if (!client.connected()) {
    reconnect();
  }
  client.loop();

  long now = millis();
  if(now - ping_time > 60000) {
    ping_time = now;
    ++ping_val;
    sprintf(cadena,"%d",ping_val);
    client.publish("clock/ping60",cadena);
    Serial.println(":status;Ping60");
  }

  // Gestió de dades canal sèrie
  if(Serial.available()) {
     Rchar=(char) Serial.read();
     if(Rchar==LF) goto noanalitza;
     if(Rchar!=CR) { sROrdre[pROrdre++]=Rchar; goto noanalitza; }
     sROrdre[pROrdre]=0;

     Serial.println(sROrdre);   // tornem el mateix que hem rebut per possible verificació

     // Analitzem cadena rebuda de la forma 
     // :pub;topic;payload
     // :sub;topic
     cmd[0]=topic[0]=payload[0]=0;
     if(sROrdre[0]==':') {
       sep1=strstr(sROrdre,";");
       if(sep1!=NULL) {
         *sep1=0;
         sep2=strstr(sep1+1,";");
         if(sep2!=NULL) {
           *sep2=0;
           strcpy(payload,sep2+1);
         }
         strcpy(topic,sep1+1);         
       }
       strcpy(cmd,&sROrdre[1]);

       // Analitzem ordre rebuda
       msg_ok=false;
       if(strcmp("publish",cmd)==0) {
         client.publish(topic,payload);
         msg_ok=true;
       }
       if(strcmp("subscribe",cmd)==0) {
         client.subscribe(topic,1);   // no es pot posar 2 ¿?
         msg_ok=true;
       }
      if(msg_ok==false) {
        Serial.println(":status;Error");          
      }       
     }

     pROrdre=0; sROrdre[pROrdre]=0;
     noanalitza:
     Rchar=0;   // cal posar alguna cosa
  }
}

